@@ -1,4 +1,4 @@
-require 'rails/railtie'
+require 'rails'
 require 'jbuilder/jbuilder_template'
 
 class Jbuilder
@@ -6,7 +6,7 @@ class Jbuilder
     initializer :jbuilder do
       ActiveSupport.on_load :action_view do
         ActionView::Template.register_template_handler :jbuilder, JbuilderHandler
-        require 'jbuilder/dependency_tracker'
+        require 'jbuilder/jbuilder_dependency_tracker'
       end
 
       if Rails::VERSION::MAJOR >= 5
@@ -17,7 +17,7 @@ class Jbuilder
         end
 
         ActiveSupport.on_load :action_controller do
-          if self == ActionController::API
+          if name == 'ActionController::API'
             include ActionController::Helpers
             include ActionController::ImplicitRender
           end
